@@ -5,9 +5,10 @@ Major TODO items:
 
 * TODO: error handling for all of the big issues (disk-space, memory, script failures)
 """
-import subprocess, os, sys, shutil, tempfile, glob
+import subprocess, os, sys, shutil, tempfile, glob, logging, traceback
 from optparse import OptionParser
-from fussy import unpack, errors
+from fussy import unpack, errors, nbio
+log = logging.getLogger( __name__ )
 
 CURRENT_LINK = 'current'
 FAILSAFE_NAME = 'failsafe'
@@ -34,7 +35,9 @@ def clean( target=DEFAULT_FIRMWARE_DIRECTORY, protected = None ):
     protected.append( current_target )
     for path in glob.glob( os.path.join( target, '*' )):
         if path not in protected:
+            log.warn( 'Removing unused firmware: %s', path )
             shutil.rmtree( path, True )
+    return current_target
 
 def final_path( link ):
     """Get the final path of the given link"""
@@ -92,6 +95,7 @@ def install_bytes( filename, keyring='/etc/fussy/keys', target='/opt/firmware' )
             'rsync', '-aq',
         ]
         if os.path.exists( current ):
+            log.info( 'Reducing firmware size with hard-link compression' )
             command.append( '--link-dest=%s'%( current,),)
         # TODO: figure out some way to configure rsync to not create a second 
         # level directory when told `rsync -a a b`
@@ -102,6 +106,7 @@ def install_bytes( filename, keyring='/etc/fussy/keys', target='/opt/firmware' )
         command.extend([
             final_target,
         ])
+        log.info( 'Fixating firmware' )
         subprocess.check_call( command )
         return final_target
     finally:
@@ -129,21 +134,32 @@ def enable( final_target, current ):
     
     try:
         if os.path.exists( pre_install ):
-            subprocess.check_call( [
+            log.info( 'Running pre-install script' )
+            def report_progress( line ):
+                log.info( 'pre-install: %s', line )
+            pipe = nbio.Process( [
                 pre_install,
                 final_target,
-            ])
+            ], by_line=True, stderr=-1) | report_progress
+            pipe()
+        log.info( 'Setting firmware current' )
         swap_link( final_target, current )
     except Exception, err:
         # we failed in either pre-setup or swapping
+        log.warn( 'Failed during pre-install or swap, aborting' )
         shutil.rmtree( final_target, True )
         raise
     
     if os.path.exists( post_install ):
-        subprocess.check_call([
+        log.info( 'Running post-install script' )
+        def report_progress( line ):
+            log.info( 'post-install: %s', line )
+        pipe = nbio.Process([
             post_install,
             final_target,
-        ])
+        ], by_line=True, stderr=-1) | report_progress
+        pipe()
+    
 
 def install( filename, keyring='/etc/fussy/keys', target='/opt/firmware' ):
     """Install given firmware <filename> into given target directory
@@ -167,18 +183,25 @@ def install( filename, keyring='/etc/fussy/keys', target='/opt/firmware' ):
             previous = None
     if not previous and os.path.exists( failsafe ) :
         previous = failsafe
-    
+    log.info( 'Previous installation: %s', previous )
+    log.info( 'Unpacking firmware to disk' )
     final_target = install_bytes( filename, keyring, target )
+    log.info( 'New installation: %s', final_target )
     assert os.path.exists( final_target )
     try:
         enable( final_target, current )
     except Exception, err:
+        log.error( "Failure installing %s: %s", err )
+        log.error( "Traceback: %s", traceback.format_exc())
         if previous:
+            log.warn( "Attempting to restore previous: %s", previous )
             enable( previous, current )
             raise errors.RevertedFailure( previous )
+        log.error( "Unable to recover, contact support!" )
         raise errors.UnrecoverableError( str(err) )
         # TODO: need lots more logic in the back-off code...
     else:
+        log.info( "Successfully installed %s", final_target )
         return final_target
 
 def get_options():
@@ -208,6 +231,12 @@ def get_options():
         type="string",
         help="Directory into which to rsync the firmware (default /opt/firmware)",
     )
+    parser.add_option(
+        '-l', '--logfile',
+        dest='logfile',
+        default='/tmp/fussy-install.log',
+        help = "File into which to write the fussy installation log (default /tmp/fussy-install.log)",
+    )
     return parser
 
 def main():
@@ -225,7 +254,20 @@ def main():
             options.file = args[0]
         else:
             parser.error( "Need a file to install" )
-    return install( options.file, options.keyring, options.target )
+    if options.logfile:
+        logging.basicConfig(filename=options.logfile,level=logging.INFO)
+    else:
+        logging.basicConfig(level=logging.INFO)
+    try:
+        return install( options.file, options.keyring, options.target )
+    except Exception, err:
+        log.error(
+            "Failure during installation: %s", err,
+        )
+        log.error(
+            "Traceback: %s", traceback.format_exc(),
+        )
+        raise
 
 def clean_main():
     """Main entry-point for fussy-clean script 
@@ -245,6 +287,5 @@ def clean_main():
         help="Directory into which to rsync the firmware (default /opt/firmware)",
     )
     options,args = parser.parse_args()
-    error_code, installed = clean( options.target )
-    # TODO: return codes to say "backed off", not just success/failure
-    return error_code
+    clean( options.target )
+    return 0
